@@ -32,7 +32,7 @@ struct HeadChunk {
 }
 
 impl Client {
-    #[instrument(level = "debug")]
+    #[instrument(skip(self), level = "debug")]
     /// Reads [`Bytes`] from the network, whose contents are contained within on or more chunks.
     pub async fn read_bytes(&self, address: BytesAddress) -> Result<Bytes> {
         let chunk = self.get_chunk(address.name()).await?;
@@ -83,7 +83,7 @@ impl Client {
         self.seek(data_map, position, length).await
     }
 
-    #[instrument(level = "trace")]
+    #[instrument(skip(self), level = "trace")]
     pub(crate) async fn get_chunk(&self, name: &XorName) -> Result<Chunk> {
         let res = self
             .send_query(DataQuery::GetChunk(ChunkAddress(*name)))
@@ -114,7 +114,7 @@ impl Client {
 
     /// Encrypts a binary large object (blob) and returns the resulting address and all chunks.
     /// Does not store anything to the network.
-    #[instrument(skip(blob), level = "trace")]
+    #[instrument(skip(self, blob), level = "trace")]
     fn encrypt_blob(&self, blob: Blob, scope: Scope) -> Result<(BytesAddress, Vec<Chunk>)> {
         let owner = encryption(scope, self.public_key());
         encrypt_blob(blob.bytes(), owner.as_ref())
@@ -140,7 +140,7 @@ impl Client {
 
     /// Directly writes [`Bytes`] to the network in the
     /// form of immutable chunks, without any batching.
-    #[instrument(skip(bytes), level = "debug")]
+    #[instrument(skip(self, bytes), level = "debug")]
     pub async fn upload(&self, bytes: Bytes, scope: Scope) -> Result<BytesAddress> {
         if let Ok(blob) = Blob::new(bytes.clone()) {
             self.upload_blob(blob, scope).await
@@ -293,23 +293,25 @@ impl Client {
 mod tests {
     use super::Spot;
 
-    use crate::client::{client_api::blob_apis::Blob, utils::test_utils::create_test_client};
+    use crate::client::{
+        client_api::blob_apis::Blob,
+        utils::test_utils::{create_test_client, init_test_logger},
+    };
     use crate::types::{utils::random_bytes, BytesAddress, Keypair};
     use crate::url::Scope;
-
     use bytes::Bytes;
     use eyre::Result;
     use futures::future::join_all;
     use rand::rngs::OsRng;
     use tokio::time::Instant;
+    use tracing::Instrument;
 
     const MIN_BLOB_SIZE: usize = self_encryption::MIN_ENCRYPTABLE_BYTES;
     const DELAY_DIVIDER: usize = 500_000;
 
     #[test]
     fn deterministic_chunking() -> Result<()> {
-        let _outer_span = tracing::info_span!("test__register_basics").entered();
-
+        init_test_logger();
         let keypair = Keypair::new_ed25519(&mut OsRng);
         let blob = random_bytes(MIN_BLOB_SIZE);
 
@@ -334,7 +336,8 @@ mod tests {
     // Test storing and reading min size blob.
     #[tokio::test(flavor = "multi_thread")]
     async fn store_and_read_3kb() -> Result<()> {
-        let _outer_span = tracing::info_span!("store_and_read_3kb").entered();
+        init_test_logger();
+        let _start_span = tracing::info_span!("store_and_read_3kb").entered();
 
         let client = create_test_client().await?;
 
@@ -357,14 +360,27 @@ mod tests {
 
         // Test storing private blob with the same value.
         // Should not conflict and return same address
-        let address = client.upload_blob(blob.clone(), Scope::Private).await?;
+        let address = client
+            .upload_blob(blob.clone(), Scope::Private)
+            .instrument(tracing::info_span!(
+                "checking no conflict on same private upload"
+            ))
+            .await?;
         assert_eq!(address, private_address);
 
         // Test storing public blob with the same value. Should not conflict.
-        let public_address = client.upload_blob(blob.clone(), Scope::Public).await?;
+        let public_address = client
+            .upload_blob(blob.clone(), Scope::Public)
+            .instrument(tracing::info_span!("checking no conflict on public upload"))
+            .await?;
+
+        assert_ne!(public_address, private_address);
 
         // Assert that the public blob is stored.
-        let read_data = client.read_bytes(public_address).await?;
+        let read_data = client
+            .read_bytes(public_address)
+            .instrument(tracing::info_span!("reading_public"))
+            .await?;
 
         compare(blob.bytes(), read_data)?;
 
@@ -373,12 +389,16 @@ mod tests {
 
     #[tokio::test(flavor = "multi_thread")]
     async fn seek_in_data() -> Result<()> {
+        init_test_logger();
         let _outer_span = tracing::info_span!("seek_in_data").entered();
-
         let client = create_test_client().await?;
+
         for i in 1..5 {
+            // let _outer_span = tracing::info_span!("seek_in_data").entered();
             let size = i * MIN_BLOB_SIZE;
+            let _outer_span = tracing::info_span!("size:", size).entered();
             for divisor in 2..5 {
+                let _outer_span = tracing::info_span!("divisor", divisor).entered();
                 let len = size / divisor;
                 let blob = Blob::new(random_bytes(size))?;
 
@@ -411,31 +431,24 @@ mod tests {
 
     #[tokio::test(flavor = "multi_thread")]
     async fn store_and_read_1kb() -> Result<()> {
-        let _outer_span = tracing::info_span!("store_and_read_1kb").entered();
-
         store_and_read_spot(MIN_BLOB_SIZE / 3, Scope::Public).await?;
         store_and_read_spot(MIN_BLOB_SIZE / 3, Scope::Private).await
     }
 
     #[tokio::test(flavor = "multi_thread")]
     async fn store_and_read_1mb() -> Result<()> {
-        let _outer_span = tracing::info_span!("store_and_read_1mb").entered();
-
         store_and_read_blob(1024 * 1024, Scope::Public).await?;
         store_and_read_blob(1024 * 1024, Scope::Private).await
     }
 
     #[tokio::test(flavor = "multi_thread")]
     async fn store_and_read_10mb() -> Result<()> {
-        let _outer_span = tracing::info_span!("store_and_read_10mb").entered();
         store_and_read_blob(10 * 1024 * 1024, Scope::Private).await
     }
 
     #[tokio::test(flavor = "multi_thread")]
     #[ignore = "too heavy for CI"]
     async fn store_and_read_20mb() -> Result<()> {
-        let _outer_span = tracing::info_span!("store_and_read_20mb").entered();
-
         store_and_read_blob(20 * 1024 * 1024, Scope::Private).await
     }
 
@@ -449,7 +462,9 @@ mod tests {
     #[tokio::test(flavor = "multi_thread")]
     #[ignore = "too heavy for CI"]
     async fn parallel_timings() -> Result<()> {
+        init_test_logger();
         let _outer_span = tracing::info_span!("parallel_timings").entered();
+
         let client = create_test_client().await?;
 
         let handles = (0..1000_usize)
@@ -482,6 +497,9 @@ mod tests {
     #[tokio::test(flavor = "multi_thread")]
     #[ignore = "too heavy for CI"]
     async fn one_by_one_timings() -> Result<()> {
+        init_test_logger();
+        let _outer_span = tracing::info_span!("test__one_by_one_timings").entered();
+
         let client = create_test_client().await?;
 
         for i in 0..1000_usize {
@@ -496,8 +514,17 @@ mod tests {
     }
 
     async fn store_and_read_blob(size: usize, scope: Scope) -> Result<()> {
+        init_test_logger();
+        // cannot use scope as var w/ macro
+        let _outer_span = if scope == Scope::Public {
+            tracing::info_span!("store_and_read_public_blob", size).entered()
+        } else {
+            tracing::info_span!("store_and_read_private_blob", size).entered()
+        };
+
         let blob = Blob::new(random_bytes(size))?;
         let client = create_test_client().await?;
+
         let address = client.upload_blob(blob.clone(), scope).await?;
 
         // the larger the file, the longer we have to wait before we start querying
@@ -513,8 +540,17 @@ mod tests {
     }
 
     async fn store_and_read_spot(size: usize, scope: Scope) -> Result<()> {
+        init_test_logger();
+        // cannot use scope as var w/ macro
+        let _outer_span = if scope == Scope::Public {
+            tracing::info_span!("store_and_read_public_spot", size).entered()
+        } else {
+            tracing::info_span!("store_and_read_private_spot", size).entered()
+        };
+
         let spot = Spot::new(random_bytes(size))?;
         let client = create_test_client().await?;
+
         let address = client.upload_spot(spot.clone(), scope).await?;
 
         // the larger the size, the longer we have to wait before we start querying
